@@ -58,15 +58,13 @@ cairo_surface_t *ev_document_misc_surface_rotate_and_scale (cairo_surface_t *sur
 void             ev_document_misc_invert_surface (cairo_surface_t *surface);
 void		 ev_document_misc_invert_pixbuf  (GdkPixbuf       *pixbuf);
 
-gdouble          ev_document_misc_get_screen_dpi (GdkScreen *screen);
+gdouble          ev_document_misc_get_screen_dpi (GdkScreen *screen, gint monitor);
 
 gchar           *ev_document_misc_format_date (GTime utime);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 void             ev_document_misc_get_pointer_position (GtkWidget *widget,
 							gint      *x,
 							gint      *y);
-#endif
 
 G_END_DECLS
 
